
/*
 * OLSR ad-hoc routing table management protocol
 * Copyright (C) 2003 Andreas Tnnesen (andreto@ifi.uio.no)
 *
 * This file is part of olsrd-unik.
 *
 * olsrd-unik is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * olsrd-unik is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with olsrd-unik; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */


#include "net.h"
#include "olsr.h"
#include <stdlib.h>

void
init_net()
{
  ptf_list = NULL;

  return;
}

/**
 *Sends a packet on a given interface.
 *
 *@param ifp the interface to send on.
 *
 *@return negative on error
 */
int
net_output(struct interface *ifp)
{
  struct sockaddr_in *sin;  
  struct sockaddr_in dst;
  struct sockaddr_in6 *sin6;  
  struct sockaddr_in6 dst6;
  struct ptf *tmp_ptf_list;
  int i, x;

  sin = NULL;
  sin6 = NULL;

  /* Add the Packet seqno */
  ((struct olsr*)packet)->olsr_seqno = htons(ifp->olsr_seqnum++);
  /* Set the packetlength */
  //((struct olsr*)packet)->olsr_packlen = htons(outputsize);

  if(ipversion == AF_INET)
    {
      /* IP version 4 */
      sin = (struct sockaddr_in *)&ifp->int_broadaddr;

      /* Copy sin */
      dst = *sin;
      sin = &dst;

      /* Set user defined broadcastaddr */
      if(bcast_set)
	memcpy(&dst.sin_addr.s_addr, &bcastaddr.sin_addr, sizeof(olsr_u32_t));

      if (sin->sin_port == 0)
	sin->sin_port = olsr_udp_port;
    }
  else
    {
      /* IP version 6 */
      sin6 = (struct sockaddr_in6 *)&ifp->int6_multaddr;
      /* Copy sin */
      dst6 = *sin6;
      sin6 = &dst6;
    }

  /*
   *if the '-disp- option was given
   *we print her decimal contetnt of the packets
   */
  /*
  if(disp_pack_out)
    {
      switch(packet[4])
	{
	case(HELLO_MESSAGE):printf("\n\tHELLO ");break;
	case(TC_MESSAGE):printf("\n\tTC ");break;
	case(MID_MESSAGE):printf("\n\tMID ");break;
	case(HNA_MESSAGE):printf("\n\tHNA ");break;
	}
      if(ipversion == AF_INET)
	printf("to %s size: %d\n\t", ip_to_string(&sin->sin_addr.s_addr), outputsize);
      else
	printf("to %s size: %d\n\t", ip6_to_string(&sin6->sin6_addr), outputsize);

      x = 0;

      for(i = 0; i < outputsize;i++)
	{
	  if(x == 4)
	    {
	      x = 0;
	      printf("\n\t");
	    }
	  x++;
	  if(ipversion == AF_INET)
	    printf(" %3i", (u_char) packet[i]);
	  else
	    printf(" %2x", (u_char) packet[i]);
	}
      
      printf("\n");
    }
  */

  /*
   *Call possible packet transform functions registered by plugins  
   */
  tmp_ptf_list = ptf_list;
  while(tmp_ptf_list != NULL)
    {
      tmp_ptf_list->function(packet, &outputsize);
      tmp_ptf_list = tmp_ptf_list->next;
    }

  /*
   *if the '-disp- option was given
   *we print her decimal contetnt of the packets
   */
  if(disp_pack_out)
    {
      switch(packet[4])
	{
	case(HELLO_MESSAGE):printf("\n\tHELLO ");break;
	case(TC_MESSAGE):printf("\n\tTC ");break;
	case(MID_MESSAGE):printf("\n\tMID ");break;
	case(HNA_MESSAGE):printf("\n\tHNA ");break;
	default:printf("\n\tTYPE: %d ", packet[4]); break;
	}
      if(ipversion == AF_INET)
	printf("to %s size: %d\n\t", ip_to_string(&sin->sin_addr.s_addr), outputsize);
      else
	printf("to %s size: %d\n\t", ip6_to_string(&sin6->sin6_addr), outputsize);

      x = 0;

      for(i = 0; i < outputsize;i++)
	{
	  if(x == 4)
	    {
	      x = 0;
	      printf("\n\t");
	    }
	  x++;
	  if(ipversion == AF_INET)
	    printf(" %3i", (u_char) packet[i]);
	  else
	    printf(" %2x", (u_char) packet[i]);
	}
      
      printf("\n");
    }

  /*
   *packet points to the same memory area as the *msg pointer
   *used when building packets.
   */
  
  if(ipversion == AF_INET)
    {
      /* IP version 4 */
      if(sendto(s[ifp->if_nr], packet, outputsize, MSG_DONTROUTE, (struct sockaddr *)sin, sizeof (*sin)) < 0)
	{
	  perror("sendto(v4)");
	  syslog(LOG_ERR, "OLSR: sendto IPv4 %m");
	  outputsize = 0;
	  return -1;
	}
    }
  else
    {
      /* IP version 6 */
      if(sendto(s[ifp->if_nr], packet, outputsize, MSG_DONTROUTE, (struct sockaddr *)sin6, sizeof (*sin6)) < 0)
	{
	  perror("sendto(v6)");
	  syslog(LOG_ERR, "OLSR: sendto IPv6 %m");
	  fprintf(stderr, "Socket: %d interface: %d\n", s[ifp->if_nr], ifp->if_nr);
	  fprintf(stderr, "To: %s (size: %d)\n", ip6_to_string(&sin6->sin6_addr), sizeof(*sin6));
	  fprintf(stderr, "Outputsize: %d\n", outputsize);
	  outputsize = 0;
	  return -1;
	}
    }
  
  outputsize = 0;

  return 1;
}






/**
 *Forward a message on all interfaces
 *
 *@return negative on error
 */
int
net_forward()
{
  struct sockaddr_in *sin;  
  struct sockaddr_in dst;
  struct sockaddr_in6 *sin6;  
  struct sockaddr_in6 dst6;
  struct interface *ifn;
  struct ptf *tmp_ptf_list;
  int i, x;
  
  sin = NULL;
  sin6 = NULL;
  
  for (ifn = ifnet; ifn; ifn = ifn->int_next) 
    {
      
      /* Add the Packet seqno */
      ((struct olsr*)fwd_packet)->olsr_seqno = htons(ifn->olsr_seqnum++);

      if(ipversion == AF_INET)
	{
	  /* IP version 4 */
	  sin = (struct sockaddr_in *)&ifn->int_broadaddr;
	  
	  /* Copy sin */
	  dst = *sin;
	  sin = &dst;

	  /* Set user defined broadcastaddr */
	  if(bcast_set)
	    memcpy(&dst.sin_addr.s_addr, &bcastaddr.sin_addr, sizeof(olsr_u32_t));
	  
	  if (sin->sin_port == 0)
	    sin->sin_port = olsr_udp_port;
	}
      else
	{
	  /* IP version 6 */
	  sin6 = (struct sockaddr_in6 *)&ifn->int6_multaddr;
	  /* Copy sin */
	  dst6 = *sin6;
	  sin6 = &dst6;
	}

      /*
       *if the '-disp- option was given
       *we print her decimal contetnt of the packets
       */
      if(disp_pack_out)
	{
	  if(ipversion == AF_INET)
	    printf("FORWARDING to %s size: %d\n\t", ip_to_string(&sin->sin_addr.s_addr), fwdsize);
	  else
	    printf("FORWARDING to %s size: %d\n\t", ip6_to_string(&sin6->sin6_addr), fwdsize);
	  
	  x = 0;
	  
	  for(i = 0; i < fwdsize;i++)
	    {
	      if(x == 4)
		{
		  x = 0;
		  printf("\n\t");
		}
	      x++;
	      if(ipversion == AF_INET)
		printf(" %3i", (u_char) fwd_packet[i]);
	      else
		printf(" %2x", (u_char) fwd_packet[i]);
	    }
	  
	  printf("\n");
	}
  
      
      /*
       *Call possible packet transform functions registered by plugins  
       */
      tmp_ptf_list = ptf_list;
      while(tmp_ptf_list != NULL)
	{
	  tmp_ptf_list->function(fwd_packet, &fwdsize);
	  tmp_ptf_list = tmp_ptf_list->next;
	}


      if(ipversion == AF_INET)
	{
	  /* IP version 4 */
	  if(sendto(s[ifn->if_nr], fwd_packet, fwdsize, MSG_DONTROUTE, (struct sockaddr *)sin, sizeof (*sin)) < 0)
	    {
	      perror("sendto(v4)");
	      syslog(LOG_ERR, "OLSR: forward sendto IPv4 %m");
	      return -1;
	    }
	}
      else
	{
	  /* IP version 6 */
	  if(sendto(s[ifn->if_nr], fwd_packet, fwdsize, MSG_DONTROUTE, (struct sockaddr *)sin6, sizeof (*sin6)) < 0)
	    {
	      perror("sendto(v6)");
	      syslog(LOG_ERR, "OLSR: forward sendto IPv6 %m");
	      fprintf(stderr, "Socket: %d interface: %d\n", s[ifn->if_nr], ifn->if_nr);
	      fprintf(stderr, "To: %s (size: %d)\n", ip6_to_string(&sin6->sin6_addr), sizeof(*sin6));
	      fprintf(stderr, "Outputsize: %d\n", fwdsize);
	      return -1;
	    }
	}

    }      
  return 1;
}

/*
 * Add a packet transform function
 */
int
add_ptf(int (*f)(char *, int *))
{

  struct ptf *new_ptf;

  new_ptf = olsr_malloc(sizeof(struct ptf), "Add PTF");

  new_ptf->next = ptf_list;
  new_ptf->function = f;

  ptf_list = new_ptf;

  return 1;
}

/*
 * Remove a packet transform function
 */
int
del_ptf(int (*f)(char *, int *))
{
  struct ptf *tmp_ptf, *prev;

  tmp_ptf = ptf_list;
  prev = NULL;

  while(tmp_ptf)
    {
      if(tmp_ptf->function == f)
	{
	  /* Remove entry */
	  if(prev == NULL)
	    {
	      ptf_list = tmp_ptf->next;
	      free(tmp_ptf);
	    }
	  else
	    {
	      prev ->next = tmp_ptf->next;
	      free(tmp_ptf);
	    }
	  return 1;
	}
      prev = tmp_ptf;
      tmp_ptf = tmp_ptf->next;
    }

  return 0;
}


int
join_mcast(struct interface *ifs, int sock)
{
  /* See linux/in6.h */

  struct ipv6_mreq mcastreq;

  COPY_IP(&mcastreq.ipv6mr_multiaddr, &ifs->int6_multaddr.sin6_addr);
  mcastreq.ipv6mr_interface = ifs->if_index;

  olsr_printf(3, "Interface %s joining multicast %s...",	ifs->int_name, olsr_ip_to_string((union olsr_ip_addr *)&ifs->int6_multaddr.sin6_addr));
  /* Send multicast */
  if(setsockopt(sock, IPPROTO_IPV6, IPV6_ADD_MEMBERSHIP, &mcastreq, sizeof(struct ipv6_mreq)) < 0)
    {
      perror("Join multicast");
      return -1;
    }


  /* Join reciever group */
  if(setsockopt(sock, IPPROTO_IPV6, IPV6_JOIN_GROUP, &mcastreq, sizeof(struct ipv6_mreq)) < 0)
    {
      perror("Join multicast send");
      return -1;
    }

  
  if(setsockopt(sock, IPPROTO_IPV6, IPV6_MULTICAST_IF, &mcastreq.ipv6mr_interface, sizeof(mcastreq.ipv6mr_interface)) < 0)
    {
      perror("Set multicast if");
      return -1;
    }


  olsr_printf(3, "OK\n");
  return 0;
}


/**
 * Create a IPv6 netmask based on a prefix length
 *
 */
int
olsr_prefix_to_netmask(union olsr_ip_addr *adr, olsr_u16_t prefix)
{
  int p, i;

  if(adr == NULL)
    return 0;

  p = prefix;
  i = 0;

  memset(adr, 0, ipsize);

  for(;p > 0; p -= 8)
    {
      adr->v6.s6_addr[i] = (p < 8) ? 0xff ^ (0xff << p) : 0xff;
      i++;
    }

#ifdef DEBUG
  olsr_printf(3, "Prefix %d = Netmask: %s\n", prefix, olsr_ip_to_string(adr));
#endif

  return 1;
}



/**
 * Calculate prefix length based on a netmask
 *
 */
olsr_u16_t
olsr_netmask_to_prefix(union olsr_ip_addr *adr)
{
  olsr_u16_t prefix;
  int i, tmp;

  prefix = 0;

  memset(adr, 0, ipsize);

  for(i = 0; i < 16; i++)
    {
      if(adr->v6.s6_addr[i] == 0xff)
	{
	  prefix += 8;
	}
      else
	{
	  for(tmp = adr->v6.s6_addr[i];
	      tmp > 0;
	      tmp = tmp >> 1)
	    prefix++;
	}
    }

#ifdef DEBUG
  olsr_printf(3, "Netmask: %s = Prefix %d\n", olsr_ip_to_string(adr), prefix);
#endif

  return prefix;
}



/**
 *Converts a sockaddr struct to a string representing
 *the IP address from the sockaddr struct
 *
 *<b>NON REENTRANT!!!!</b>
 *
 *@param address_to_convert the sockaddr struct to "convert"
 *@return a char pointer to the string containing the IP
 */
char *
sockaddr_to_string(struct sockaddr *address_to_convert)
{
  struct sockaddr_in           *address;
  
  address=(struct sockaddr_in *)address_to_convert; 
  return(inet_ntoa(address->sin_addr));
  
}


/**
 *Converts the 32bit olsr_u32_t datatype to
 *a char array.
 *
 *<b>NON REENTRANT!!!!</b>
 *
 *@param address the olsr_u32_t to "convert"
 *@return a char pointer to the string containing the IP
 */

char *
ip_to_string(olsr_u32_t *address)
{

  struct in_addr in;
  in.s_addr=*address;
  return(inet_ntoa(in));
  
}




/**
 *Converts the 32bit olsr_u32_t datatype to
 *a char array.
 *
 *<b>NON REENTRANT!!!!</b>
 *
 *@param addr6 the address to "convert"
 *@return a char pointer to the string containing the IP
 */

char *
ip6_to_string(struct in6_addr *addr6)
{
  return (char *)inet_ntop(AF_INET6, addr6, ipv6_buf, sizeof(ipv6_buf));
}


char *
olsr_ip_to_string(union olsr_ip_addr *addr)
{

  char *ret;
  struct in_addr in;
  
  if(ipversion == AF_INET)
    {
      in.s_addr=addr->v4;
      ret = inet_ntoa(in);
    }
  else
    {
      /* IPv6 */
      ret = (char *)inet_ntop(AF_INET6, &addr->v6, ipv6_buf, sizeof(ipv6_buf));
    }

  return ret;
}
